@@ -28,7 +28,6 @@ def _is_versioning_col(col):
 
 
 def _history_mapper(local_mapper):
-
     cls = local_mapper.class_
 
     if cls.__dict__.get("_history_mapper_configured", False):
@@ -114,7 +113,7 @@ def _history_mapper(local_mapper):
             Column(
                 "changed",
                 DateTime,
-                default=datetime.datetime.utcnow,
+                default=lambda: datetime.datetime.now(datetime.timezone.utc),
                 info=version_meta,
             )
         )
