@@ -1,9 +1,11 @@
 # mysql/mariadbconnector.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
+
 
 """
 
@@ -32,7 +34,6 @@ import re
 from .base import MySQLCompiler
 from .base import MySQLDialect
 from .base import MySQLExecutionContext
-from .base import MySQLIdentifierPreparer
 from ... import sql
 from ... import util
 
@@ -40,21 +41,26 @@ mariadb_cpy_minimum_version = (1, 0, 1)
 
 
 class MySQLExecutionContext_mariadbconnector(MySQLExecutionContext):
+    _lastrowid = None
+
     def create_server_side_cursor(self):
         return self._dbapi_connection.cursor(buffered=False)
 
     def create_default_cursor(self):
         return self._dbapi_connection.cursor(buffered=True)
 
+    def post_exec(self):
+        if self.isinsert and self.compiled.postfetch_lastrowid:
+            self._lastrowid = self.cursor.lastrowid
+
+    def get_lastrowid(self):
+        return self._lastrowid
+
 
 class MySQLCompiler_mariadbconnector(MySQLCompiler):
     pass
 
 
-class MySQLIdentifierPreparer_mariadbconnector(MySQLIdentifierPreparer):
-    pass
-
-
 class MySQLDialect_mariadbconnector(MySQLDialect):
     driver = "mariadbconnector"
     supports_statement_cache = True
@@ -78,7 +84,6 @@ class MySQLDialect_mariadbconnector(MySQLDialect):
     default_paramstyle = "qmark"
     execution_ctx_cls = MySQLExecutionContext_mariadbconnector
     statement_compiler = MySQLCompiler_mariadbconnector
-    preparer = MySQLIdentifierPreparer_mariadbconnector
 
     supports_server_side_cursors = True
 
@@ -97,7 +102,7 @@ class MySQLDialect_mariadbconnector(MySQLDialect):
             return (99, 99, 99)
 
     def __init__(self, **kwargs):
-        super(MySQLDialect_mariadbconnector, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.paramstyle = "qmark"
         if self.dbapi is not None:
             if self._dbapi_version < mariadb_cpy_minimum_version:
@@ -108,13 +113,11 @@ class MySQLDialect_mariadbconnector(MySQLDialect):
                 )
 
     @classmethod
-    def dbapi(cls):
+    def import_dbapi(cls):
         return __import__("mariadb")
 
     def is_disconnect(self, e, connection, cursor):
-        if super(MySQLDialect_mariadbconnector, self).is_disconnect(
-            e, connection, cursor
-        ):
+        if super().is_disconnect(e, connection, cursor):
             return True
         elif isinstance(e, self.dbapi.Error):
             str_e = str(e).lower()
@@ -169,24 +172,21 @@ class MySQLDialect_mariadbconnector(MySQLDialect):
     def _detect_charset(self, connection):
         return "utf8mb4"
 
-    _isolation_lookup = set(
-        [
+    def get_isolation_level_values(self, dbapi_connection):
+        return (
             "SERIALIZABLE",
             "READ UNCOMMITTED",
             "READ COMMITTED",
             "REPEATABLE READ",
             "AUTOCOMMIT",
-        ]
-    )
+        )
 
-    def _set_isolation_level(self, connection, level):
+    def set_isolation_level(self, connection, level):
         if level == "AUTOCOMMIT":
             connection.autocommit = True
         else:
             connection.autocommit = False
-            super(MySQLDialect_mariadbconnector, self)._set_isolation_level(
-                connection, level
-            )
+            super().set_isolation_level(connection, level)
 
     def do_begin_twophase(self, connection, xid):
         connection.execute(
