@@ -88,6 +88,7 @@ connection URI::
         "postgresql+psycopg2://scott:tiger@192.168.0.199:5432/test?sslmode=require"
     )
 
+
 Unix Domain Connections
 ------------------------
 
@@ -104,6 +105,19 @@ using ``host`` as an additional keyword argument::
 
     create_engine("postgresql+psycopg2://user:password@/dbname?host=/var/lib/postgresql")
 
+.. warning::  The format accepted here allows for a hostname in the main URL
+   in addition to the "host" query string argument.  **When using this URL
+   format, the initial host is silently ignored**.  That is, this URL::
+
+        engine = create_engine("postgresql+psycopg2://user:password@myhost1/dbname?host=myhost2")
+
+   Above, the hostname ``myhost1`` is **silently ignored and discarded.**  The
+   host which is connected is the ``myhost2`` host.
+
+   This is to maintain some degree of compatibility with PostgreSQL's own URL
+   format which has been tested to behave the same way and for which tools like
+   PifPaf hardcode two hostnames.
+
 .. seealso::
 
     `PQconnectdbParams \
@@ -627,6 +641,13 @@ class PGDialect_psycopg2(_PGDialect_common_psycopg):
     ):
         _PGDialect_common_psycopg.__init__(self, **kwargs)
 
+        if self._native_inet_types:
+            raise NotImplementedError(
+                "The psycopg2 dialect does not implement "
+                "ipaddress type handling; native_inet_types cannot be set "
+                "to ``True`` when using this dialect."
+            )
+
         # Parse executemany_mode argument, allowing it to be only one of the
         # symbol names
         self.executemany_mode = parse_user_argument_for_enum(
@@ -804,7 +825,6 @@ class PGDialect_psycopg2(_PGDialect_common_psycopg):
 
     @util.memoized_instancemethod
     def _hstore_oids(self, dbapi_connection):
-
         extras = self._psycopg2_extras
         oids = extras.HstoreAdapter.get_oids(dbapi_connection)
         if oids is not None and oids[0]:
