@@ -699,10 +699,6 @@ lookup is used instead:
   ``REAL``, ``FLOA`` or ``DOUB``.
 * Otherwise, the :class:`_types.NUMERIC` type is used.
 
-.. versionadded:: 0.9.3 Support for SQLite type affinity rules when reflecting
-   columns.
-
-
 .. _sqlite_partial_index:
 
 Partial Indexes
@@ -720,8 +716,6 @@ The index will be rendered at create time as::
     CREATE INDEX test_idx1 ON testtbl (data)
     WHERE data > 5 AND data < 10
 
-.. versionadded:: 0.9.9
-
 .. _sqlite_dotted_column_names:
 
 Dotted Column Names
@@ -733,12 +727,6 @@ databases in general, as the dot is a syntactically significant character,
 the SQLite driver up until version **3.10.0** of SQLite has a bug which
 requires that SQLAlchemy filter out these dots in result sets.
 
-.. versionchanged:: 1.1
-
-    The following SQLite issue has been resolved as of version 3.10.0
-    of SQLite.  SQLAlchemy as of **1.1** automatically disables its internal
-    workarounds based on detection of this version.
-
 The bug, entirely outside of SQLAlchemy, can be illustrated thusly::
 
     import sqlite3
@@ -955,8 +943,6 @@ class _DateTimeMixin:
         the type will generate its DDL as DATE_CHAR, DATETIME_CHAR,
         TIME_CHAR.
 
-        .. versionadded:: 1.0.0
-
         """
         spec = self._storage_format % {
             "year": 0,
@@ -1464,7 +1450,6 @@ class SQLiteCompiler(compiler.SQLCompiler):
         return target_text
 
     def visit_on_conflict_do_nothing(self, on_conflict, **kw):
-
         target_text = self._on_conflict_target(on_conflict, **kw)
 
         if target_text:
@@ -1542,7 +1527,6 @@ class SQLiteCompiler(compiler.SQLCompiler):
 
 class SQLiteDDLCompiler(compiler.DDLCompiler):
     def get_column_specification(self, column, **kwargs):
-
         coltype = self.dialect.type_compiler_instance.process(
             column.type, type_expression=column
         )
@@ -1664,7 +1648,6 @@ class SQLiteDDLCompiler(compiler.DDLCompiler):
         return text
 
     def visit_foreign_key_constraint(self, constraint, **kw):
-
         local_table = constraint.elements[0].parent.table
         remote_table = constraint.elements[0].column.table
 
@@ -1910,7 +1893,10 @@ class SQLiteDialect(default.DefaultDialect):
     supports_default_values = True
     supports_default_metavalue = False
 
+    # sqlite issue:
     # https://github.com/python/cpython/issues/93421
+    # note this parameter is no longer used by the ORM or default dialect
+    # see #9414
     supports_sane_rowcount_returning = False
 
     supports_empty_insert = False
@@ -2274,7 +2260,6 @@ class SQLiteDialect(default.DefaultDialect):
         persisted,
         tablesql,
     ):
-
         if generated:
             # the type of a column "cc INTEGER GENERATED ALWAYS AS (1 + 42)"
             # somehow is "INTEGER GENERATED ALWAYS"
@@ -2552,7 +2537,6 @@ class SQLiteDialect(default.DefaultDialect):
     def get_unique_constraints(
         self, connection, table_name, schema=None, **kw
     ):
-
         auto_index_by_sig = {}
         for idx in self.get_indexes(
             connection,
