@@ -1,14 +1,17 @@
 # sqlalchemy/events.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """Core event interfaces."""
 
+from __future__ import annotations
+
 from .engine.events import ConnectionEvents
 from .engine.events import DialectEvents
+from .pool import PoolResetState
 from .pool.events import PoolEvents
 from .sql.base import SchemaEventTarget
 from .sql.events import DDLEvents
