@@ -1,9 +1,12 @@
 # sqlalchemy/ext/baked.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
+
+
 """Baked query extension.
 
 Provides a creational pattern for the :class:`.query.Query` object which
@@ -13,24 +16,23 @@ compiled result to be fully cached.
 
 """
 
+import collections.abc as collections_abc
 import logging
 
 from .. import exc as sa_exc
 from .. import util
 from ..orm import exc as orm_exc
-from ..orm import strategy_options
 from ..orm.query import Query
 from ..orm.session import Session
 from ..sql import func
 from ..sql import literal_column
 from ..sql import util as sql_util
-from ..util import collections_abc
 
 
 log = logging.getLogger(__name__)
 
 
-class Bakery(object):
+class Bakery:
     """Callable which returns a :class:`.BakedQuery`.
 
     This object is returned by the class method
@@ -52,7 +54,7 @@ class Bakery(object):
         return self.cls(self.cache, initial_fn, args)
 
 
-class BakedQuery(object):
+class BakedQuery:
     """A builder object for :class:`.query.Query` objects."""
 
     __slots__ = "steps", "_bakery", "_cache_key", "_spoiled"
@@ -309,7 +311,7 @@ class BakedQuery(object):
         return query
 
 
-class Result(object):
+class Result:
     """Invokes a :class:`.BakedQuery` against a :class:`.Session`.
 
     The :class:`_baked.Result` object is where the actual :class:`.query.Query`
@@ -428,12 +430,10 @@ class Result(object):
         Note this uses a subquery to ensure an accurate count regardless
         of the structure of the original statement.
 
-        .. versionadded:: 1.1.6
-
         """
 
         col = func.count(literal_column("*"))
-        bq = self.bq.with_criteria(lambda q: q._from_self(col))
+        bq = self.bq.with_criteria(lambda q: q._legacy_from_self(col))
         return bq.for_session(self.session).params(self._params).scalar()
 
     def scalar(self):
@@ -443,8 +443,6 @@ class Result(object):
 
         Equivalent to :meth:`_query.Query.scalar`.
 
-        .. versionadded:: 1.1.6
-
         """
         try:
             ret = self.one()
@@ -484,8 +482,6 @@ class Result(object):
 
         Equivalent to :meth:`_query.Query.one_or_none`.
 
-        .. versionadded:: 1.0.9
-
         """
         return self._iter().one_or_none()
 
@@ -523,15 +519,13 @@ class Result(object):
             # None present in ident - turn those comparisons
             # into "IS NULL"
             if None in primary_key_identity:
-                nones = set(
-                    [
-                        _get_params[col].key
-                        for col, value in zip(
-                            mapper.primary_key, primary_key_identity
-                        )
-                        if value is None
-                    ]
-                )
+                nones = {
+                    _get_params[col].key
+                    for col, value in zip(
+                        mapper.primary_key, primary_key_identity
+                    )
+                    if value is None
+                }
                 _lcl_get_clause = sql_util.adapt_criterion_to_null(
                     _lcl_get_clause, nones
                 )
@@ -560,14 +554,12 @@ class Result(object):
             setup, tuple(elem is None for elem in primary_key_identity)
         )
 
-        params = dict(
-            [
-                (_get_params[primary_key].key, id_val)
-                for id_val, primary_key in zip(
-                    primary_key_identity, mapper.primary_key
-                )
-            ]
-        )
+        params = {
+            _get_params[primary_key].key: id_val
+            for id_val, primary_key in zip(
+                primary_key_identity, mapper.primary_key
+            )
+        }
 
         result = list(bq.for_session(self.session).params(**params))
         l = len(result)
@@ -579,70 +571,4 @@ class Result(object):
             return None
 
 
-@util.deprecated(
-    "1.2", "Baked lazy loading is now the default implementation."
-)
-def bake_lazy_loaders():
-    """Enable the use of baked queries for all lazyloaders systemwide.
-
-    The "baked" implementation of lazy loading is now the sole implementation
-    for the base lazy loader; this method has no effect except for a warning.
-
-    """
-    pass
-
-
-@util.deprecated(
-    "1.2", "Baked lazy loading is now the default implementation."
-)
-def unbake_lazy_loaders():
-    """Disable the use of baked queries for all lazyloaders systemwide.
-
-    This method now raises NotImplementedError() as the "baked" implementation
-    is the only lazy load implementation.  The
-    :paramref:`_orm.relationship.bake_queries` flag may be used to disable
-    the caching of queries on a per-relationship basis.
-
-    """
-    raise NotImplementedError(
-        "Baked lazy loading is now the default implementation"
-    )
-
-
-@strategy_options.loader_option()
-def baked_lazyload(loadopt, attr):
-    """Indicate that the given attribute should be loaded using "lazy"
-    loading with a "baked" query used in the load.
-
-    """
-    return loadopt.set_relationship_strategy(attr, {"lazy": "baked_select"})
-
-
-@baked_lazyload._add_unbound_fn
-@util.deprecated(
-    "1.2",
-    "Baked lazy loading is now the default "
-    "implementation for lazy loading.",
-)
-def baked_lazyload(*keys):
-    return strategy_options._UnboundLoad._from_keys(
-        strategy_options._UnboundLoad.baked_lazyload, keys, False, {}
-    )
-
-
-@baked_lazyload._add_unbound_all_fn
-@util.deprecated(
-    "1.2",
-    "Baked lazy loading is now the default "
-    "implementation for lazy loading.",
-)
-def baked_lazyload_all(*keys):
-    return strategy_options._UnboundLoad._from_keys(
-        strategy_options._UnboundLoad.baked_lazyload, keys, True, {}
-    )
-
-
-baked_lazyload = baked_lazyload._unbound_fn
-baked_lazyload_all = baked_lazyload_all._unbound_all_fn
-
 bakery = BakedQuery.bakery
