@@ -1,18 +1,30 @@
 # ext/serializer.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
 
 """Serializer/Deserializer objects for usage with SQLAlchemy query structures,
 allowing "contextual" deserialization.
 
+.. legacy::
+
+    The serializer extension is **legacy** and should not be used for
+    new development.
+
 Any SQLAlchemy query structure, either based on sqlalchemy.sql.*
 or sqlalchemy.orm.* can be used.  The mappers, Tables, Columns, Session
 etc. which are referenced by the structure are not persisted in serialized
 form, but are instead re-associated with the query structure
 when it is deserialized.
 
+.. warning:: The serializer extension uses pickle to serialize and
+   deserialize objects, so the same security consideration mentioned
+   in the `python documentation
+   <https://docs.python.org/3/library/pickle.html>`_ apply.
+
 Usage is nearly the same as that of the standard Python pickle module::
 
     from sqlalchemy.ext.serializer import loads, dumps
@@ -21,7 +33,8 @@ Usage is nearly the same as that of the standard Python pickle module::
 
     # ... define mappers
 
-    query = Session.query(MyClass).filter(MyClass.somedata=='foo').order_by(MyClass.sortkey)
+    query = Session.query(MyClass).
+        filter(MyClass.somedata=='foo').order_by(MyClass.sortkey)
 
     # pickle the query
     serialized = dumps(query)
@@ -51,51 +64,63 @@ needed for:
 
 """
 
-from ..orm import class_mapper
-from ..orm.session import Session
-from ..orm.mapper import Mapper
-from ..orm.interfaces import MapperProperty
-from ..orm.attributes import QueryableAttribute
-from .. import Table, Column
-from ..engine import Engine
-from ..util import pickle, byte_buffer, b64encode, b64decode, text_type
+from io import BytesIO
+import pickle
 import re
 
+from .. import Column
+from .. import Table
+from ..engine import Engine
+from ..orm import class_mapper
+from ..orm.interfaces import MapperProperty
+from ..orm.mapper import Mapper
+from ..orm.session import Session
+from ..util import b64decode
+from ..util import b64encode
 
-__all__ = ['Serializer', 'Deserializer', 'dumps', 'loads']
+
+__all__ = ["Serializer", "Deserializer", "dumps", "loads"]
 
 
 def Serializer(*args, **kw):
     pickler = pickle.Pickler(*args, **kw)
 
     def persistent_id(obj):
-        #print "serializing:", repr(obj)
-        if isinstance(obj, QueryableAttribute):
-            cls = obj.impl.class_
-            key = obj.impl.key
-            id = "attribute:" + key + ":" + b64encode(pickle.dumps(cls))
-        elif isinstance(obj, Mapper) and not obj.non_primary:
-            id = "mapper:" + b64encode(pickle.dumps(obj.class_))
+        # print "serializing:", repr(obj)
+        if isinstance(obj, Mapper) and not obj.non_primary:
+            id_ = "mapper:" + b64encode(pickle.dumps(obj.class_))
         elif isinstance(obj, MapperProperty) and not obj.parent.non_primary:
-            id = "mapperprop:" + b64encode(pickle.dumps(obj.parent.class_)) + \
-                                    ":" + obj.key
+            id_ = (
+                "mapperprop:"
+                + b64encode(pickle.dumps(obj.parent.class_))
+                + ":"
+                + obj.key
+            )
         elif isinstance(obj, Table):
-            id = "table:" + text_type(obj.key)
+            if "parententity" in obj._annotations:
+                id_ = "mapper_selectable:" + b64encode(
+                    pickle.dumps(obj._annotations["parententity"].class_)
+                )
+            else:
+                id_ = f"table:{obj.key}"
         elif isinstance(obj, Column) and isinstance(obj.table, Table):
-            id = "column:" + text_type(obj.table.key) + ":" + text_type(obj.key)
+            id_ = f"column:{obj.table.key}:{obj.key}"
         elif isinstance(obj, Session):
-            id = "session:"
+            id_ = "session:"
         elif isinstance(obj, Engine):
-            id = "engine:"
+            id_ = "engine:"
         else:
             return None
-        return id
+        return id_
 
     pickler.persistent_id = persistent_id
     return pickler
 
+
 our_ids = re.compile(
-            r'(mapperprop|mapper|table|column|session|attribute|engine):(.*)')
+    r"(mapperprop|mapper|mapper_selectable|table|column|"
+    r"session|attribute|engine):(.*)"
+)
 
 
 def Deserializer(file, metadata=None, scoped_session=None, engine=None):
@@ -111,27 +136,30 @@ def Deserializer(file, metadata=None, scoped_session=None, engine=None):
         else:
             return None
 
-    def persistent_load(id):
-        m = our_ids.match(text_type(id))
+    def persistent_load(id_):
+        m = our_ids.match(str(id_))
         if not m:
             return None
         else:
             type_, args = m.group(1, 2)
-            if type_ == 'attribute':
+            if type_ == "attribute":
                 key, clsarg = args.split(":")
                 cls = pickle.loads(b64decode(clsarg))
                 return getattr(cls, key)
             elif type_ == "mapper":
                 cls = pickle.loads(b64decode(args))
                 return class_mapper(cls)
+            elif type_ == "mapper_selectable":
+                cls = pickle.loads(b64decode(args))
+                return class_mapper(cls).__clause_element__()
             elif type_ == "mapperprop":
-                mapper, keyname = args.split(':')
+                mapper, keyname = args.split(":")
                 cls = pickle.loads(b64decode(mapper))
                 return class_mapper(cls).attrs[keyname]
             elif type_ == "table":
                 return metadata.tables[args]
             elif type_ == "column":
-                table, colname = args.split(':')
+                table, colname = args.split(":")
                 return metadata.tables[table].c[colname]
             elif type_ == "session":
                 return scoped_session()
@@ -139,18 +167,19 @@ def Deserializer(file, metadata=None, scoped_session=None, engine=None):
                 return get_engine()
             else:
                 raise Exception("Unknown token: %s" % type_)
+
     unpickler.persistent_load = persistent_load
     return unpickler
 
 
-def dumps(obj, protocol=0):
-    buf = byte_buffer()
+def dumps(obj, protocol=pickle.HIGHEST_PROTOCOL):
+    buf = BytesIO()
     pickler = Serializer(buf, protocol)
     pickler.dump(obj)
     return buf.getvalue()
 
 
 def loads(data, metadata=None, scoped_session=None, engine=None):
-    buf = byte_buffer(data)
+    buf = BytesIO(data)
     unpickler = Deserializer(buf, metadata, scoped_session, engine)
     return unpickler.load()
