@@ -149,7 +149,6 @@ def _bulk_insert(
         bookkeeping = False
 
     for table, super_mapper in mappers_to_run:
-
         # find bindparams in the statement. For bulk, we don't really know if
         # a key in the params applies to a different table since we are
         # potentially inserting for multiple tables here; looking at the
@@ -368,7 +367,6 @@ class ORMDMLState(AbstractORMCompileState):
     def _get_orm_crud_kv_pairs(
         cls, mapper, statement, kv_iterator, needs_to_be_cacheable
     ):
-
         core_get_crud_kv_pairs = UpdateDMLState._get_crud_kv_pairs
 
         for k, v in kv_iterator:
@@ -522,7 +520,6 @@ class ORMDMLState(AbstractORMCompileState):
         """
 
         if orm_level_statement._returning:
-
             fs = FromStatement(
                 orm_level_statement._returning,
                 dml_level_statement,
@@ -578,7 +575,6 @@ class ORMDMLState(AbstractORMCompileState):
         bind_arguments,
         result,
     ):
-
         execution_context = result.context
         compile_state = execution_context.compiled.compile_state
 
@@ -638,7 +634,6 @@ class BulkUDCompileState(ORMDMLState):
         bind_arguments,
         is_pre_event,
     ):
-
         (
             update_options,
             execution_options,
@@ -661,11 +656,13 @@ class BulkUDCompileState(ORMDMLState):
         except KeyError:
             assert False, "statement had 'orm' plugin but no plugin_subject"
         else:
-            bind_arguments["mapper"] = plugin_subject.mapper
+            if plugin_subject:
+                bind_arguments["mapper"] = plugin_subject.mapper
+                update_options += {"_subject_mapper": plugin_subject.mapper}
 
-        update_options += {"_subject_mapper": plugin_subject.mapper}
-
-        if not isinstance(params, list):
+        if "parententity" not in statement.table._annotations:
+            update_options += {"_dml_strategy": "core_only"}
+        elif not isinstance(params, list):
             if update_options._dml_strategy == "auto":
                 update_options += {"_dml_strategy": "orm"}
             elif update_options._dml_strategy == "bulk":
@@ -695,7 +692,6 @@ class BulkUDCompileState(ORMDMLState):
                 session._autoflush()
 
             if update_options._dml_strategy == "orm":
-
                 if update_options._synchronize_session == "auto":
                     update_options = cls._do_pre_synchronize_auto(
                         session,
@@ -758,7 +754,6 @@ class BulkUDCompileState(ORMDMLState):
         bind_arguments,
         result,
     ):
-
         # this stage of the execution is called after the
         # do_orm_execute event hook.  meaning for an extension like
         # horizontal sharding, this step happens *within* the horizontal
@@ -1005,7 +1000,6 @@ class BulkUDCompileState(ORMDMLState):
         bind_arguments,
         update_options,
     ):
-
         try:
             eval_condition = cls._eval_condition_from_statement(
                 update_options, statement
@@ -1155,7 +1149,6 @@ class BulkORMInsert(ORMDMLState, InsertDMLState):
         bind_arguments,
         is_pre_event,
     ):
-
         (
             insert_options,
             execution_options,
@@ -1171,9 +1164,9 @@ class BulkORMInsert(ORMDMLState, InsertDMLState):
         except KeyError:
             assert False, "statement had 'orm' plugin but no plugin_subject"
         else:
-            bind_arguments["mapper"] = plugin_subject.mapper
-
-        insert_options += {"_subject_mapper": plugin_subject.mapper}
+            if plugin_subject:
+                bind_arguments["mapper"] = plugin_subject.mapper
+                insert_options += {"_subject_mapper": plugin_subject.mapper}
 
         if not params:
             if insert_options._dml_strategy == "auto":
@@ -1224,7 +1217,6 @@ class BulkORMInsert(ORMDMLState, InsertDMLState):
         bind_arguments: _BindArguments,
         conn: Connection,
     ) -> _result.Result:
-
         insert_options = execution_options.get(
             "_sa_orm_insert_options", cls.default_insert_options
         )
@@ -1306,7 +1298,6 @@ class BulkORMInsert(ORMDMLState, InsertDMLState):
 
     @classmethod
     def create_for_statement(cls, statement, compiler, **kw) -> BulkORMInsert:
-
         self = cast(
             BulkORMInsert,
             super().create_for_statement(statement, compiler, **kw),
@@ -1390,16 +1381,23 @@ class BulkORMInsert(ORMDMLState, InsertDMLState):
 class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
     @classmethod
     def create_for_statement(cls, statement, compiler, **kw):
-
         self = cls.__new__(cls)
 
         dml_strategy = statement._annotations.get(
             "dml_strategy", "unspecified"
         )
 
-        if dml_strategy == "bulk":
+        toplevel = not compiler.stack
+
+        if toplevel and dml_strategy == "bulk":
             self._setup_for_bulk_update(statement, compiler)
-        elif dml_strategy in ("orm", "unspecified"):
+        elif (
+            dml_strategy == "core_only"
+            or dml_strategy == "unspecified"
+            and "parententity" not in statement.table._annotations
+        ):
+            UpdateDMLState.__init__(self, statement, compiler, **kw)
+        elif not toplevel or dml_strategy in ("orm", "unspecified"):
             self._setup_for_orm_update(statement, compiler)
 
         return self
@@ -1407,6 +1405,8 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
     def _setup_for_orm_update(self, statement, compiler, **kw):
         orm_level_statement = statement
 
+        toplevel = not compiler.stack
+
         ext_info = statement.table._annotations["parententity"]
 
         self.mapper = mapper = ext_info.mapper
@@ -1416,8 +1416,8 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
         self._init_global_attributes(
             statement,
             compiler,
-            toplevel=True,
-            process_criteria_for_toplevel=True,
+            toplevel=toplevel,
+            process_criteria_for_toplevel=toplevel,
         )
 
         if statement._values:
@@ -1451,9 +1451,12 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
 
         use_supplemental_cols = False
 
-        synchronize_session = compiler._annotations.get(
-            "synchronize_session", None
-        )
+        if not toplevel:
+            synchronize_session = None
+        else:
+            synchronize_session = compiler._annotations.get(
+                "synchronize_session", None
+            )
         can_use_returning = compiler._annotations.get(
             "can_use_returning", None
         )
@@ -1486,13 +1489,14 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
                 *(list(mapper.local_table.primary_key))
             )
 
-        new_stmt = self._setup_orm_returning(
-            compiler,
-            orm_level_statement,
-            new_stmt,
-            dml_mapper=mapper,
-            use_supplemental_cols=use_supplemental_cols,
-        )
+        if toplevel:
+            new_stmt = self._setup_orm_returning(
+                compiler,
+                orm_level_statement,
+                new_stmt,
+                dml_mapper=mapper,
+                use_supplemental_cols=use_supplemental_cols,
+            )
 
         self.statement = new_stmt
 
@@ -1542,15 +1546,19 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
         bind_arguments: _BindArguments,
         conn: Connection,
     ) -> _result.Result:
-
         update_options = execution_options.get(
             "_sa_orm_update_options", cls.default_update_options
         )
 
-        if update_options._dml_strategy not in ("orm", "auto", "bulk"):
+        if update_options._dml_strategy not in (
+            "orm",
+            "auto",
+            "bulk",
+            "core_only",
+        ):
             raise sa_exc.ArgumentError(
                 "Valid strategies for ORM UPDATE strategy "
-                "are 'orm', 'auto', 'bulk'"
+                "are 'orm', 'auto', 'bulk', 'core_only'"
             )
 
         result: _result.Result[Any]
@@ -1615,7 +1623,6 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
         is_delete_using: bool = False,
         is_executemany: bool = False,
     ) -> bool:
-
         # normal answer for "should we use RETURNING" at all.
         normal_answer = (
             dialect.update_returning and mapper.local_table.implicit_returning
@@ -1689,7 +1696,6 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
     def _do_post_synchronize_evaluate(
         cls, session, statement, result, update_options
     ):
-
         matched_objects = cls._get_matched_objects_on_criteria(
             update_options,
             session.identity_map.all_states(),
@@ -1785,7 +1791,6 @@ class BulkORMUpdate(BulkUDCompileState, UpdateDMLState):
 
         states = set()
         for obj, state, dict_ in matched_objects:
-
             to_evaluate = state.unmodified.intersection(evaluated_keys)
 
             for key in to_evaluate:
@@ -1814,6 +1819,20 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
     def create_for_statement(cls, statement, compiler, **kw):
         self = cls.__new__(cls)
 
+        dml_strategy = statement._annotations.get(
+            "dml_strategy", "unspecified"
+        )
+
+        if (
+            dml_strategy == "core_only"
+            or dml_strategy == "unspecified"
+            and "parententity" not in statement.table._annotations
+        ):
+            DeleteDMLState.__init__(self, statement, compiler, **kw)
+            return self
+
+        toplevel = not compiler.stack
+
         orm_level_statement = statement
 
         ext_info = statement.table._annotations["parententity"]
@@ -1822,8 +1841,8 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
         self._init_global_attributes(
             statement,
             compiler,
-            toplevel=True,
-            process_criteria_for_toplevel=True,
+            toplevel=toplevel,
+            process_criteria_for_toplevel=toplevel,
         )
 
         new_stmt = statement._clone()
@@ -1841,9 +1860,12 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
 
         use_supplemental_cols = False
 
-        synchronize_session = compiler._annotations.get(
-            "synchronize_session", None
-        )
+        if not toplevel:
+            synchronize_session = None
+        else:
+            synchronize_session = compiler._annotations.get(
+                "synchronize_session", None
+            )
         can_use_returning = compiler._annotations.get(
             "can_use_returning", None
         )
@@ -1870,13 +1892,14 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
 
             new_stmt = new_stmt.return_defaults(*new_stmt.table.primary_key)
 
-        new_stmt = self._setup_orm_returning(
-            compiler,
-            orm_level_statement,
-            new_stmt,
-            dml_mapper=mapper,
-            use_supplemental_cols=use_supplemental_cols,
-        )
+        if toplevel:
+            new_stmt = self._setup_orm_returning(
+                compiler,
+                orm_level_statement,
+                new_stmt,
+                dml_mapper=mapper,
+                use_supplemental_cols=use_supplemental_cols,
+            )
 
         self.statement = new_stmt
 
@@ -1892,7 +1915,6 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
         bind_arguments: _BindArguments,
         conn: Connection,
     ) -> _result.Result:
-
         update_options = execution_options.get(
             "_sa_orm_update_options", cls.default_update_options
         )
@@ -1905,12 +1927,10 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
                 "session.connection().execute(stmt, parameters)"
             )
 
-        if update_options._dml_strategy not in (
-            "orm",
-            "auto",
-        ):
+        if update_options._dml_strategy not in ("orm", "auto", "core_only"):
             raise sa_exc.ArgumentError(
-                "Valid strategies for ORM DELETE strategy are 'orm', 'auto'"
+                "Valid strategies for ORM DELETE strategy are 'orm', 'auto', "
+                "'core_only'"
             )
 
         return super().orm_execute_statement(
@@ -1928,7 +1948,6 @@ class BulkORMDelete(BulkUDCompileState, DeleteDMLState):
         is_delete_using: bool = False,
         is_executemany: bool = False,
     ) -> bool:
-
         # normal answer for "should we use RETURNING" at all.
         normal_answer = (
             dialect.delete_returning and mapper.local_table.implicit_returning
