@@ -55,6 +55,7 @@ from .properties import MappedColumn
 from .util import _extract_mapped_subtype
 from .util import _is_mapped_annotation
 from .util import class_mapper
+from .util import de_stringify_annotation
 from .. import event
 from .. import exc
 from .. import util
@@ -64,7 +65,6 @@ from ..sql.schema import Column
 from ..sql.schema import Table
 from ..util import topological
 from ..util.typing import _AnnotationScanType
-from ..util.typing import de_stringify_annotation
 from ..util.typing import is_fwd_ref
 from ..util.typing import is_literal
 from ..util.typing import Protocol
@@ -126,6 +126,7 @@ class _DataclassArguments(TypedDict):
     unsafe_hash: Union[_NoArg, bool]
     match_args: Union[_NoArg, bool]
     kw_only: Union[_NoArg, bool]
+    dataclass_callable: Union[_NoArg, Callable[..., Type[Any]]]
 
 
 def _declared_mapping_info(
@@ -210,13 +211,13 @@ def _get_immediate_cls_attr(
         return getattr(cls, attrname)
 
     for base in cls.__mro__[1:]:
-        _is_classicial_inherits = _dive_for_cls_manager(base) is not None
+        _is_classical_inherits = _dive_for_cls_manager(base) is not None
 
         if attrname in base.__dict__ and (
             base is cls
             or (
                 (base in cls.__bases__ if strict else True)
-                and not _is_classicial_inherits
+                and not _is_classical_inherits
             )
         ):
             return getattr(base, attrname)
@@ -241,7 +242,6 @@ def _dive_for_cls_manager(cls: Type[_O]) -> Optional[ClassManager[_O]]:
 def _as_declarative(
     registry: _RegistryType, cls: Type[Any], dict_: _ClassDict
 ) -> Optional[_MapperConfig]:
-
     # declarative scans the class for attributes.  no table or mapper
     # args passed separately.
     return _MapperConfig.setup_mapping(registry, cls, dict_, None, {})
@@ -259,9 +259,9 @@ def _mapper(
 
 @util.preload_module("sqlalchemy.orm.decl_api")
 def _is_declarative_props(obj: Any) -> bool:
-    declared_attr = util.preloaded.orm_decl_api.declared_attr
+    _declared_attr_common = util.preloaded.orm_decl_api._declared_attr_common
 
-    return isinstance(obj, (declared_attr, util.classproperty))
+    return isinstance(obj, (_declared_attr_common, util.classproperty))
 
 
 def _check_declared_props_nocascade(
@@ -358,7 +358,6 @@ class _MapperConfig:
                 )
 
     def set_cls_attribute(self, attrname: str, value: _T) -> _T:
-
         manager = instrumentation.manager_of_class(self.cls)
         manager.install_member(attrname, value)
         return value
@@ -439,6 +438,7 @@ class _CollectedAnnotation(NamedTuple):
     is_dataclass: bool
     attr_value: Any
     originating_module: str
+    originating_class: Type[Any]
 
 
 class _ClassScanMapperConfig(_MapperConfig):
@@ -450,12 +450,14 @@ class _ClassScanMapperConfig(_MapperConfig):
         "local_table",
         "persist_selectable",
         "declared_columns",
+        "column_ordering",
         "column_copies",
         "table_args",
         "tablename",
         "mapper_args",
         "mapper_args_fn",
         "inherits",
+        "single",
         "allow_dataclass_fields",
         "dataclass_setup_arguments",
         "is_dataclass_prior_to_mapping",
@@ -470,6 +472,7 @@ class _ClassScanMapperConfig(_MapperConfig):
     local_table: Optional[FromClause]
     persist_selectable: Optional[FromClause]
     declared_columns: util.OrderedSet[Column[Any]]
+    column_ordering: Dict[Column[Any], int]
     column_copies: Dict[
         Union[MappedColumn[Any], Column[Any]],
         Union[MappedColumn[Any], Column[Any]],
@@ -479,6 +482,7 @@ class _ClassScanMapperConfig(_MapperConfig):
     table_args: Optional[_TableArgsType]
     mapper_args_fn: Optional[Callable[[], Dict[str, Any]]]
     inherits: Optional[Type[Any]]
+    single: bool
 
     is_dataclass_prior_to_mapping: bool
     allow_unmapped_annotations: bool
@@ -508,7 +512,6 @@ class _ClassScanMapperConfig(_MapperConfig):
         table: Optional[FromClause],
         mapper_kw: _MapperKwArgs,
     ):
-
         # grab class dict before the instrumentation manager has been added.
         # reduces cycles
         self.clsdict_view = (
@@ -521,8 +524,9 @@ class _ClassScanMapperConfig(_MapperConfig):
         self.collected_attributes = {}
         self.collected_annotations = {}
         self.declared_columns = util.OrderedSet()
+        self.column_ordering = {}
         self.column_copies = {}
-
+        self.single = False
         self.dataclass_setup_arguments = dca = getattr(
             self.cls, "_sa_apply_dc_transforms", None
         )
@@ -612,10 +616,9 @@ class _ClassScanMapperConfig(_MapperConfig):
         if not sa_dataclass_metadata_key:
 
             def attribute_is_overridden(key: str, obj: Any) -> bool:
-                return getattr(cls, key) is not obj
+                return getattr(cls, key, obj) is not obj
 
         else:
-
             all_datacls_fields = {
                 f.name: f.metadata[sa_dataclass_metadata_key]
                 for f in util.dataclass_fields(cls)
@@ -712,9 +715,9 @@ class _ClassScanMapperConfig(_MapperConfig):
 
         if not sa_dataclass_metadata_key:
 
-            def local_attributes_for_class() -> Iterable[
-                Tuple[str, Any, Any, bool]
-            ]:
+            def local_attributes_for_class() -> (
+                Iterable[Tuple[str, Any, Any, bool]]
+            ):
                 return (
                     (
                         name,
@@ -732,9 +735,9 @@ class _ClassScanMapperConfig(_MapperConfig):
 
             fixed_sa_dataclass_metadata_key = sa_dataclass_metadata_key
 
-            def local_attributes_for_class() -> Iterable[
-                Tuple[str, Any, Any, bool]
-            ]:
+            def local_attributes_for_class() -> (
+                Iterable[Tuple[str, Any, Any, bool]]
+            ):
                 for name in names:
                     field = dataclass_fields.get(name, None)
                     if field and sa_dataclass_metadata_key in field.metadata:
@@ -803,7 +806,6 @@ class _ClassScanMapperConfig(_MapperConfig):
             local_attributes_for_class,
             locally_collected_columns,
         ) in bases:
-
             # this transfer can also take place as we scan each name
             # for finer-grained control of how collected_attributes is
             # populated, as this is what impacts column ordering.
@@ -861,7 +863,7 @@ class _ClassScanMapperConfig(_MapperConfig):
                         # should only be __table__
                         continue
                 elif class_mapped:
-                    if _is_declarative_props(obj):
+                    if _is_declarative_props(obj) and not obj._quiet:
                         util.warn(
                             "Regular (i.e. not __special__) "
                             "attribute '%s.%s' uses @declared_attr, "
@@ -969,7 +971,10 @@ class _ClassScanMapperConfig(_MapperConfig):
                         # then test if this name is already handled and
                         # otherwise proceed to generate.
                         if not fixed_table:
-                            assert name in collected_attributes
+                            assert (
+                                name in collected_attributes
+                                or attribute_is_overridden(name, None)
+                            )
                         continue
                     else:
                         # here, the attribute is some other kind of
@@ -1031,7 +1036,6 @@ class _ClassScanMapperConfig(_MapperConfig):
         self.mapper_args_fn = mapper_args_fn
 
     def _setup_dataclasses_transforms(self) -> None:
-
         dataclass_setup_arguments = self.dataclass_setup_arguments
         if not dataclass_setup_arguments:
             return
@@ -1047,6 +1051,19 @@ class _ClassScanMapperConfig(_MapperConfig):
                 "'@registry.mapped_as_dataclass'"
             )
 
+        warn_for_non_dc_attrs = collections.defaultdict(list)
+
+        def _allow_dataclass_field(
+            key: str, originating_class: Type[Any]
+        ) -> bool:
+            if (
+                originating_class is not self.cls
+                and "__dataclass_fields__" not in originating_class.__dict__
+            ):
+                warn_for_non_dc_attrs[originating_class].append(key)
+
+            return True
+
         manager = instrumentation.manager_of_class(self.cls)
         assert manager is not None
 
@@ -1070,20 +1087,45 @@ class _ClassScanMapperConfig(_MapperConfig):
                     is_dc,
                     attr_value,
                     originating_module,
+                    originating_class,
                 ) in self.collected_annotations.items()
-                if key not in self.collected_attributes
-                # issue #9226; check for attributes that we've collected
-                # which are already instrumented, which we would assume
-                # mean we are in an ORM inheritance mapping and this attribute
-                # is already mapped on the superclass.   Under no circumstance
-                # should any QueryableAttribute be sent to the dataclass()
-                # function; anything that's mapped should be Field and
-                # that's it
-                or not isinstance(
-                    self.collected_attributes[key], QueryableAttribute
+                if _allow_dataclass_field(key, originating_class)
+                and (
+                    key not in self.collected_attributes
+                    # issue #9226; check for attributes that we've collected
+                    # which are already instrumented, which we would assume
+                    # mean we are in an ORM inheritance mapping and this
+                    # attribute is already mapped on the superclass.   Under
+                    # no circumstance should any QueryableAttribute be sent to
+                    # the dataclass() function; anything that's mapped should
+                    # be Field and that's it
+                    or not isinstance(
+                        self.collected_attributes[key], QueryableAttribute
+                    )
                 )
             )
         ]
+
+        if warn_for_non_dc_attrs:
+            for (
+                originating_class,
+                non_dc_attrs,
+            ) in warn_for_non_dc_attrs.items():
+                util.warn_deprecated(
+                    f"When transforming {self.cls} to a dataclass, "
+                    f"attribute(s) "
+                    f"{', '.join(repr(key) for key in non_dc_attrs)} "
+                    f"originates from superclass "
+                    f"{originating_class}, which is not a dataclass.  This "
+                    f"usage is deprecated and will raise an error in "
+                    f"SQLAlchemy 2.1.  When declaring SQLAlchemy Declarative "
+                    f"Dataclasses, ensure that all mixin classes and other "
+                    f"superclasses which include attributes are also a "
+                    f"subclass of MappedAsDataclass.",
+                    "2.0",
+                    code="dcmx",
+                )
+
         annotations = {}
         defaults = {}
         for item in field_list:
@@ -1099,26 +1141,88 @@ class _ClassScanMapperConfig(_MapperConfig):
         for k, v in defaults.items():
             setattr(self.cls, k, v)
 
-        self.cls.__annotations__ = annotations
-
         self._apply_dataclasses_to_any_class(
-            dataclass_setup_arguments, self.cls
+            dataclass_setup_arguments, self.cls, annotations
         )
 
+    @classmethod
+    def _update_annotations_for_non_mapped_class(
+        cls, klass: Type[_O]
+    ) -> Mapping[str, _AnnotationScanType]:
+        cls_annotations = util.get_annotations(klass)
+
+        new_anno = {}
+        for name, annotation in cls_annotations.items():
+            if _is_mapped_annotation(annotation, klass, klass):
+                extracted = _extract_mapped_subtype(
+                    annotation,
+                    klass,
+                    klass.__module__,
+                    name,
+                    type(None),
+                    required=False,
+                    is_dataclass_field=False,
+                    expect_mapped=False,
+                )
+                if extracted:
+                    inner, _ = extracted
+                    new_anno[name] = inner
+            else:
+                new_anno[name] = annotation
+        return new_anno
+
     @classmethod
     def _apply_dataclasses_to_any_class(
-        cls, dataclass_setup_arguments: _DataclassArguments, klass: Type[_O]
+        cls,
+        dataclass_setup_arguments: _DataclassArguments,
+        klass: Type[_O],
+        use_annotations: Mapping[str, _AnnotationScanType],
     ) -> None:
         cls._assert_dc_arguments(dataclass_setup_arguments)
 
-        dataclasses.dataclass(
-            klass,
-            **{
-                k: v
-                for k, v in dataclass_setup_arguments.items()
-                if v is not _NoArg.NO_ARG
-            },
-        )
+        dataclass_callable = dataclass_setup_arguments["dataclass_callable"]
+        if dataclass_callable is _NoArg.NO_ARG:
+            dataclass_callable = dataclasses.dataclass
+
+        restored: Optional[Any]
+
+        if use_annotations:
+            # apply constructed annotations that should look "normal" to a
+            # dataclasses callable, based on the fields present.  This
+            # means remove the Mapped[] container and ensure all Field
+            # entries have an annotation
+            restored = getattr(klass, "__annotations__", None)
+            klass.__annotations__ = cast("Dict[str, Any]", use_annotations)
+        else:
+            restored = None
+
+        try:
+            dataclass_callable(
+                klass,
+                **{
+                    k: v
+                    for k, v in dataclass_setup_arguments.items()
+                    if v is not _NoArg.NO_ARG and k != "dataclass_callable"
+                },
+            )
+        except (TypeError, ValueError) as ex:
+            raise exc.InvalidRequestError(
+                f"Python dataclasses error encountered when creating "
+                f"dataclass for {klass.__name__!r}: "
+                f"{ex!r}. Please refer to Python dataclasses "
+                "documentation for additional information.",
+                code="dcte",
+            ) from ex
+        finally:
+            # restore original annotations outside of the dataclasses
+            # process; for mixins and __abstract__ superclasses, SQLAlchemy
+            # Declarative will need to see the Mapped[] container inside the
+            # annotations in order to map subclasses
+            if use_annotations:
+                if restored is None:
+                    del klass.__annotations__
+                else:
+                    klass.__annotations__ = restored
 
     @classmethod
     def _assert_dc_arguments(cls, arguments: _DataclassArguments) -> None:
@@ -1130,6 +1234,7 @@ class _ClassScanMapperConfig(_MapperConfig):
             "unsafe_hash",
             "kw_only",
             "match_args",
+            "dataclass_callable",
         }
         disallowed_args = set(arguments).difference(allowed)
         if disallowed_args:
@@ -1146,7 +1251,6 @@ class _ClassScanMapperConfig(_MapperConfig):
         expect_mapped: Optional[bool],
         attr_value: Any,
     ) -> Optional[_CollectedAnnotation]:
-
         if name in self.collected_annotations:
             return self.collected_annotations[name]
 
@@ -1211,17 +1315,21 @@ class _ClassScanMapperConfig(_MapperConfig):
             is_dataclass,
             attr_value,
             originating_class.__module__,
+            originating_class,
         )
         return ca
 
     def _warn_for_decl_attributes(
         self, cls: Type[Any], key: str, c: Any
     ) -> None:
-        if isinstance(c, expression.ColumnClause):
+        if isinstance(c, expression.ColumnElement):
             util.warn(
                 f"Attribute '{key}' on class {cls} appears to "
-                "be a non-schema 'sqlalchemy.sql.column()' "
-                "object; this won't be part of the declarative mapping"
+                "be a non-schema SQLAlchemy expression "
+                "object; this won't be part of the declarative mapping. "
+                "To map arbitrary expressions, use ``column_property()`` "
+                "or a similar function such as ``deferred()``, "
+                "``query_expression()`` etc. "
             )
 
     def _produce_column_copies(
@@ -1245,6 +1353,16 @@ class _ClassScanMapperConfig(_MapperConfig):
                 and obj is None
                 and _is_mapped_annotation(annotation, cls, originating_class)
             ):
+                # obj is None means this is the annotation only path
+
+                if attribute_is_overridden(name, obj):
+                    # perform same "overridden" check as we do for
+                    # Column/MappedColumn, this is how a mixin col is not
+                    # applied to an inherited subclass that does not have
+                    # the mixin.   the anno-only path added here for
+                    # #9564
+                    continue
+
                 collected_annotation = self._collect_annotation(
                     name, annotation, originating_class, True, obj
                 )
@@ -1260,7 +1378,6 @@ class _ClassScanMapperConfig(_MapperConfig):
                 setattr(cls, name, obj)
 
             elif isinstance(obj, (Column, MappedColumn)):
-
                 if attribute_is_overridden(name, obj):
                     # if column has been overridden
                     # (like by the InstrumentedAttribute of the
@@ -1325,7 +1442,6 @@ class _ClassScanMapperConfig(_MapperConfig):
         look_for_dataclass_things = bool(self.dataclass_setup_arguments)
 
         for k in list(collected_attributes):
-
             if k in _include_dunders:
                 continue
 
@@ -1405,8 +1521,9 @@ class _ClassScanMapperConfig(_MapperConfig):
                         is_dataclass,
                         attr_value,
                         originating_module,
+                        originating_class,
                     ) = self.collected_annotations.get(
-                        k, (None, None, None, False, None, None)
+                        k, (None, None, None, False, None, None, None)
                     )
 
                     # issue #8692 - don't do any annotation interpretation if
@@ -1442,7 +1559,6 @@ class _ClassScanMapperConfig(_MapperConfig):
                         assert expect_annotations_wo_mapped
 
                 if isinstance(value, _DCAttributeOptions):
-
                     if (
                         value._has_dataclass_arguments
                         and not look_for_dataclass_things
@@ -1500,11 +1616,11 @@ class _ClassScanMapperConfig(_MapperConfig):
 
         # extract columns from the class dict
         declared_columns = self.declared_columns
+        column_ordering = self.column_ordering
         name_to_prop_key = collections.defaultdict(set)
 
         for key, c in list(our_stuff.items()):
             if isinstance(c, _MapsColumns):
-
                 mp_to_assign = c.mapper_property_to_assign
                 if mp_to_assign:
                     our_stuff[key] = mp_to_assign
@@ -1513,11 +1629,17 @@ class _ClassScanMapperConfig(_MapperConfig):
                     # this is a MappedColumn that will produce a Column for us
                     del our_stuff[key]
 
-                for col in c.columns_to_assign:
+                for col, sort_order in c.columns_to_assign:
                     if not isinstance(c, CompositeProperty):
                         name_to_prop_key[col.name].add(key)
                     declared_columns.add(col)
 
+                    # we would assert this, however we want the below
+                    # warning to take effect instead.  See #9630
+                    # assert col not in column_ordering
+
+                    column_ordering[col] = sort_order
+
                     # if this is a MappedColumn and the attribute key we
                     # have is not what the column has for its key, map the
                     # Column explicitly under the attribute key name.
@@ -1556,6 +1678,7 @@ class _ClassScanMapperConfig(_MapperConfig):
         table_args = self.table_args
         clsdict_view = self.clsdict_view
         declared_columns = self.declared_columns
+        column_ordering = self.column_ordering
 
         manager = attributes.manager_of_class(cls)
 
@@ -1569,7 +1692,6 @@ class _ClassScanMapperConfig(_MapperConfig):
                 table_cls = Table
 
             if tablename is not None:
-
                 args: Tuple[Any, ...] = ()
                 table_kw: Dict[str, Any] = {}
 
@@ -1590,12 +1712,17 @@ class _ClassScanMapperConfig(_MapperConfig):
                 if autoload:
                     table_kw["autoload"] = True
 
+                sorted_columns = sorted(
+                    declared_columns,
+                    key=lambda c: column_ordering.get(c, 0),
+                )
                 table = self.set_cls_attribute(
                     "__table__",
                     table_cls(
                         tablename,
                         self._metadata_for_cls(manager),
-                        *(tuple(declared_columns) + tuple(args)),
+                        *sorted_columns,
+                        *args,
                         **table_kw,
                     ),
                 )
@@ -1648,6 +1775,10 @@ class _ClassScanMapperConfig(_MapperConfig):
 
         self.inherits = inherits
 
+        clsdict_view = self.clsdict_view
+        if "__table__" not in clsdict_view and self.tablename is None:
+            self.single = True
+
     def _setup_inheriting_columns(self, mapper_kw: _MapperKwArgs) -> None:
         table = self.local_table
         cls = self.cls
@@ -1659,7 +1790,6 @@ class _ClassScanMapperConfig(_MapperConfig):
             and self.inherits is None
             and not _get_immediate_cls_attr(cls, "__no_table__")
         ):
-
             raise exc.InvalidRequestError(
                 "Class %r does not have a __table__ or __tablename__ "
                 "specified and does not inherit from an existing "
@@ -1941,7 +2071,7 @@ def _add_attribute(
             mapped_cls.__mapper__.add_property(key, value)
         elif isinstance(value, _MapsColumns):
             mp = value.mapper_property_to_assign
-            for col in value.columns_to_assign:
+            for col, _ in value.columns_to_assign:
                 _undefer_column_name(key, col)
                 _table_or_raise(mapped_cls).append_column(
                     col, replace_existing=True
