@@ -1,9 +1,11 @@
 # orm/unitofwork.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
+
 
 """The internals for the unit of work system.
 
@@ -13,6 +15,14 @@ organizes them in order of dependency, and executes.
 
 """
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Dict
+from typing import Optional
+from typing import Set
+from typing import TYPE_CHECKING
+
 from . import attributes
 from . import exc as orm_exc
 from . import util as orm_util
@@ -21,16 +31,13 @@ from .. import util
 from ..util import topological
 
 
-def _warn_for_cascade_backrefs(state, prop):
-    util.warn_deprecated_20(
-        '"%s" object is being merged into a Session along the backref '
-        'cascade path for relationship "%s"; in SQLAlchemy 2.0, this '
-        "reverse cascade will not take place.  Set cascade_backrefs to "
-        "False in either the relationship() or backref() function for "
-        "the 2.0 behavior; or to set globally for the whole "
-        "Session, set the future=True flag" % (state.class_.__name__, prop),
-        code="s9r1",
-    )
+if TYPE_CHECKING:
+    from .dependency import DependencyProcessor
+    from .interfaces import MapperProperty
+    from .mapper import Mapper
+    from .session import Session
+    from .session import SessionTransaction
+    from .state import InstanceState
 
 
 def track_cascade_events(descriptor, prop):
@@ -40,7 +47,7 @@ def track_cascade_events(descriptor, prop):
     """
     key = prop.key
 
-    def append(state, item, initiator):
+    def append(state, item, initiator, **kw):
         # process "save_update" cascade rules for when
         # an instance is appended to the list of another instance
 
@@ -57,18 +64,13 @@ def track_cascade_events(descriptor, prop):
 
             if (
                 prop._cascade.save_update
-                and (
-                    (prop.cascade_backrefs and not sess.future)
-                    or key == initiator.key
-                )
+                and (key == initiator.key)
                 and not sess._contains_state(item_state)
             ):
-                if key != initiator.key:
-                    _warn_for_cascade_backrefs(item_state, prop)
                 sess._save_or_update_state(item_state)
         return item
 
-    def remove(state, item, initiator):
+    def remove(state, item, initiator, **kw):
         if item is None:
             return
 
@@ -102,7 +104,7 @@ def track_cascade_events(descriptor, prop):
                     # item
                     item_state._orphaned_outside_of_session = True
 
-    def set_(state, newvalue, oldvalue, initiator):
+    def set_(state, newvalue, oldvalue, initiator, **kw):
         # process "save_update" cascade rules for when an instance
         # is attached to another instance
         if oldvalue is newvalue:
@@ -110,7 +112,6 @@ def track_cascade_events(descriptor, prop):
 
         sess = state.session
         if sess:
-
             if sess._warn_on_events:
                 sess._flush_warning("related attribute set")
 
@@ -119,14 +120,9 @@ def track_cascade_events(descriptor, prop):
                 newvalue_state = attributes.instance_state(newvalue)
                 if (
                     prop._cascade.save_update
-                    and (
-                        (prop.cascade_backrefs and not sess.future)
-                        or key == initiator.key
-                    )
+                    and (key == initiator.key)
                     and not sess._contains_state(newvalue_state)
                 ):
-                    if key != initiator.key:
-                        _warn_for_cascade_backrefs(newvalue_state, prop)
                     sess._save_or_update_state(newvalue_state)
 
             if (
@@ -144,13 +140,28 @@ def track_cascade_events(descriptor, prop):
                     sess.expunge(oldvalue)
         return newvalue
 
-    event.listen(descriptor, "append", append, raw=True, retval=True)
-    event.listen(descriptor, "remove", remove, raw=True, retval=True)
-    event.listen(descriptor, "set", set_, raw=True, retval=True)
+    event.listen(
+        descriptor, "append_wo_mutation", append, raw=True, include_key=True
+    )
+    event.listen(
+        descriptor, "append", append, raw=True, retval=True, include_key=True
+    )
+    event.listen(
+        descriptor, "remove", remove, raw=True, retval=True, include_key=True
+    )
+    event.listen(
+        descriptor, "set", set_, raw=True, retval=True, include_key=True
+    )
 
 
-class UOWTransaction(object):
-    def __init__(self, session):
+class UOWTransaction:
+    session: Session
+    transaction: SessionTransaction
+    attributes: Dict[str, Any]
+    deps: util.defaultdict[Mapper[Any], Set[DependencyProcessor]]
+    mappers: util.defaultdict[Mapper[Any], Set[InstanceState[Any]]]
+
+    def __init__(self, session: Session):
         self.session = session
 
         # dictionary used by external actors to
@@ -294,13 +305,13 @@ class UOWTransaction(object):
 
     def register_object(
         self,
-        state,
-        isdelete=False,
-        listonly=False,
-        cancel_delete=False,
-        operation=None,
-        prop=None,
-    ):
+        state: InstanceState[Any],
+        isdelete: bool = False,
+        listonly: bool = False,
+        cancel_delete: bool = False,
+        operation: Optional[str] = None,
+        prop: Optional[MapperProperty] = None,
+    ) -> bool:
         if not self.session._contains_state(state):
             # this condition is normal when objects are registered
             # as part of a relationship cascade operation.  it should
@@ -399,9 +410,9 @@ class UOWTransaction(object):
         if cycles:
             # if yes, break the per-mapper actions into
             # per-state actions
-            convert = dict(
-                (rec, set(rec.per_state_flush_actions(self))) for rec in cycles
-            )
+            convert = {
+                rec: set(rec.per_state_flush_actions(self)) for rec in cycles
+            }
 
             # rewrite the existing dependencies to point to
             # the per-state actions for those per-mapper actions
@@ -423,11 +434,11 @@ class UOWTransaction(object):
                     for dep in convert[edge[1]]:
                         self.dependencies.add((edge[0], dep))
 
-        return set(
-            [a for a in self.postsort_actions.values() if not a.disabled]
-        ).difference(cycles)
+        return {
+            a for a in self.postsort_actions.values() if not a.disabled
+        }.difference(cycles)
 
-    def execute(self):
+    def execute(self) -> None:
         postsort_actions = self._generate_actions()
 
         postsort_actions = sorted(
@@ -454,7 +465,7 @@ class UOWTransaction(object):
             for rec in topological.sort(self.dependencies, postsort_actions):
                 rec.execute(self)
 
-    def finalize_flush_changes(self):
+    def finalize_flush_changes(self) -> None:
         """Mark processed objects as clean / deleted after a successful
         flush().
 
@@ -466,9 +477,9 @@ class UOWTransaction(object):
             return
 
         states = set(self.states)
-        isdel = set(
+        isdel = {
             s for (s, (isdelete, listonly)) in self.states.items() if isdelete
-        )
+        }
         other = states.difference(isdel)
         if isdel:
             self.session._remove_newly_deleted(isdel)
@@ -476,7 +487,9 @@ class UOWTransaction(object):
             self.session._register_persistent(other)
 
 
-class IterateMappersMixin(object):
+class IterateMappersMixin:
+    __slots__ = ()
+
     def _mappers(self, uow):
         if self.fromparent:
             return iter(
@@ -538,7 +551,7 @@ class Preprocess(IterateMappersMixin):
             return False
 
 
-class PostSortRec(object):
+class PostSortRec:
     __slots__ = ("disabled",)
 
     def __new__(cls, uow, *args):
