@@ -335,6 +335,7 @@ def expect(
     apply_propagate_attrs: Optional[ClauseElement] = None,
     argname: Optional[str] = None,
     post_inspect: bool = False,
+    disable_inspection: bool = False,
     **kw: Any,
 ) -> Any:
     if (
@@ -373,7 +374,6 @@ def expect(
         if impl._resolve_literal_only:
             resolved = impl._literal_coercion(element, **kw)
         else:
-
             original_element = element
 
             is_clause_element = False
@@ -399,7 +399,7 @@ def expect(
                         break
 
             if not is_clause_element:
-                if impl._use_inspection:
+                if impl._use_inspection and not disable_inspection:
                     insp = inspection.inspect(element, raiseerr=False)
                     if insp is not None:
                         if post_inspect:
@@ -802,7 +802,6 @@ class ExpressionElementImpl(_ColumnCoercions, RoleImpl):
 
 
 class BinaryElementImpl(ExpressionElementImpl, RoleImpl):
-
     __slots__ = ()
 
     def _literal_coercion(
@@ -961,7 +960,6 @@ class StrAsPlainColumnImpl(_CoerceLiterals, RoleImpl):
 
 
 class ByOfImpl(_CoerceLiterals, _ColumnCoercions, RoleImpl, roles.ByOfRole):
-
     __slots__ = ()
 
     _coerce_consts = True
@@ -1053,7 +1051,6 @@ class TruncatedLabelImpl(_StringOnly, RoleImpl):
 
 
 class DDLExpressionImpl(_Deannotate, _CoerceLiterals, RoleImpl):
-
     __slots__ = ()
 
     _coerce_consts = True
