@@ -11,6 +11,7 @@
 
 from __future__ import annotations
 
+from enum import Enum
 from types import ModuleType
 import typing
 from typing import Any
@@ -37,6 +38,7 @@ from .visitors import Visitable
 from .. import exc
 from .. import util
 from ..util.typing import Protocol
+from ..util.typing import Self
 from ..util.typing import TypedDict
 from ..util.typing import TypeGuard
 
@@ -67,8 +69,14 @@ _CT = TypeVar("_CT", bound=Any)
 
 _MatchedOnType = Union["GenericProtocol[Any]", NewType, Type[Any]]
 
-# replace with pep-673 when applicable
-SelfTypeEngine = typing.TypeVar("SelfTypeEngine", bound="TypeEngine[Any]")
+
+class _NoValueInList(Enum):
+    NO_VALUE_IN_LIST = 0
+    """indicates we are trying to determine the type of an expression
+    against an empty list."""
+
+
+_NO_VALUE_IN_LIST = _NoValueInList.NO_VALUE_IN_LIST
 
 
 class _LiteralProcessorType(Protocol[_T_co]):
@@ -86,6 +94,11 @@ class _ResultProcessorType(Protocol[_T_co]):
         ...
 
 
+class _SentinelProcessorType(Protocol[_T_co]):
+    def __call__(self, value: Any) -> Optional[_T_co]:
+        ...
+
+
 class _BaseTypeMemoDict(TypedDict):
     impl: TypeEngine[Any]
     result: Dict[Any, Optional[_ResultProcessorType[Any]]]
@@ -94,6 +107,7 @@ class _BaseTypeMemoDict(TypedDict):
 class _TypeMemoDict(_BaseTypeMemoDict, total=False):
     literal: Optional[_LiteralProcessorType[Any]]
     bind: Optional[_BindProcessorType[Any]]
+    sentinel: Optional[_SentinelProcessorType[Any]]
     custom: Dict[Any, object]
 
 
@@ -284,16 +298,13 @@ class TypeEngine(Visitable, Generic[_T]):
 
         :meth:`.TypeEngine.evaluates_none`
 
-    .. versionadded:: 1.1
-
-
     """
 
     _variant_mapping: util.immutabledict[
         str, TypeEngine[Any]
     ] = util.EMPTY_DICT
 
-    def evaluates_none(self: SelfTypeEngine) -> SelfTypeEngine:
+    def evaluates_none(self) -> Self:
         """Return a copy of this type which has the
         :attr:`.should_evaluate_none` flag set to True.
 
@@ -329,8 +340,6 @@ class TypeEngine(Visitable, Generic[_T]):
             ``None``
             still means "no default".
 
-        .. versionadded:: 1.1
-
         .. seealso::
 
             :ref:`session_forcing_null` - in the ORM documentation
@@ -345,7 +354,7 @@ class TypeEngine(Visitable, Generic[_T]):
         typ.should_evaluate_none = True
         return typ
 
-    def copy(self: SelfTypeEngine, **kw: Any) -> SelfTypeEngine:
+    def copy(self, **kw: Any) -> Self:
         return self.adapt(self.__class__)
 
     def compare_against_backend(
@@ -372,8 +381,6 @@ class TypeEngine(Visitable, Generic[_T]):
 
         :param conn_type: the type object reflected from the backend.
 
-        .. versionadded:: 1.0.3
-
         """
         return None
 
@@ -597,6 +604,18 @@ class TypeEngine(Visitable, Generic[_T]):
         """
         return None
 
+    def _sentinel_value_resolver(
+        self, dialect: Dialect
+    ) -> Optional[_SentinelProcessorType[_T]]:
+        """Return an optional callable that will match parameter values
+        (post-bind processing) to result values
+        (pre-result-processing), for use in the "sentinel" feature.
+
+        .. versionadded:: 2.0.10
+
+        """
+        return None
+
     @util.memoized_property
     def _has_bind_expression(self) -> bool:
         """memoized boolean, check if bind_expression is implemented.
@@ -646,10 +665,10 @@ class TypeEngine(Visitable, Generic[_T]):
         raise NotImplementedError()
 
     def with_variant(
-        self: SelfTypeEngine,
+        self,
         type_: _TypeEngineArgument[Any],
         *dialect_names: str,
-    ) -> SelfTypeEngine:
+    ) -> Self:
         r"""Produce a copy of this type object that will utilize the given
         type when applied to the dialect of the given name.
 
@@ -711,9 +730,7 @@ class TypeEngine(Visitable, Generic[_T]):
         )
         return new_type
 
-    def _resolve_for_literal(
-        self: SelfTypeEngine, value: Any
-    ) -> SelfTypeEngine:
+    def _resolve_for_literal(self, value: Any) -> Self:
         """adjust this type given a literal Python value that will be
         stored in a bound parameter.
 
@@ -731,11 +748,11 @@ class TypeEngine(Visitable, Generic[_T]):
         return self
 
     def _resolve_for_python_type(
-        self: SelfTypeEngine,
+        self,
         python_type: Type[Any],
         matched_on: _MatchedOnType,
         matched_on_flattened: Type[Any],
-    ) -> Optional[SelfTypeEngine]:
+    ) -> Optional[Self]:
         """given a Python type (e.g. ``int``, ``str``, etc. ) return an
         instance of this :class:`.TypeEngine` that's appropriate for this type.
 
@@ -946,6 +963,18 @@ class TypeEngine(Visitable, Generic[_T]):
         d["result"][coltype] = rp
         return rp
 
+    def _cached_sentinel_value_processor(
+        self, dialect: Dialect
+    ) -> Optional[_SentinelProcessorType[_T]]:
+        try:
+            return dialect._type_memos[self]["sentinel"]
+        except KeyError:
+            pass
+
+        d = self._dialect_info(dialect)
+        d["sentinel"] = bp = d["impl"]._sentinel_value_resolver(dialect)
+        return bp
+
     def _cached_custom_processor(
         self, dialect: Dialect, key: str, fn: Callable[[TypeEngine[_T]], _O]
     ) -> _O:
@@ -1084,7 +1113,6 @@ class TypeEngine(Visitable, Generic[_T]):
 
     @util.preload_module("sqlalchemy.engine.default")
     def _default_dialect(self) -> Dialect:
-
         default = util.preloaded.engine_default
 
         # dmypy / mypy seems to sporadically keep thinking this line is
@@ -1358,10 +1386,6 @@ class UserDefinedType(
     introspection is used to check for this in order to support legacy
     forms of this function.
 
-    .. versionadded:: 1.0.0 the owning expression is passed to
-       the ``get_col_spec()`` method via the keyword argument
-       ``type_expression``, if it receives ``**kw`` in its signature.
-
     The :attr:`.UserDefinedType.cache_ok` class-level flag indicates if this
     custom :class:`.UserDefinedType` is safe to be used as part of a cache key.
     This flag defaults to ``None`` which will initially generate a warning
@@ -1456,6 +1480,8 @@ class Emulated(TypeEngineMixin):
                 # as only the default logic is implemented.
                 return cls.adapt_native_to_emulated(self, **kw)
         else:
+            # this would be, both classes are Enum, or both classes
+            # are postgresql.ENUM
             if issubclass(cls, self.__class__):
                 return self.adapt_to_emulated(cls, **kw)
             else:
@@ -1495,7 +1521,6 @@ class NativeForEmulated(TypeEngineMixin):
         impl: Union[TypeEngine[Any], TypeEngineMixin],
         **kw: Any,
     ) -> TypeEngine[Any]:
-
         """Given an impl, adapt this type's class to the impl assuming
         "native".
 
@@ -1515,9 +1540,6 @@ class NativeForEmulated(TypeEngineMixin):
     #        ...
 
 
-SelfTypeDecorator = TypeVar("SelfTypeDecorator", bound="TypeDecorator[Any]")
-
-
 class TypeDecorator(SchemaEventTarget, ExternalType, TypeEngine[_T]):
     """Allows the creation of types which add additional functionality
     to an existing type.
@@ -2003,7 +2025,6 @@ class TypeDecorator(SchemaEventTarget, ExternalType, TypeEngine[_T]):
         if process_literal_param is not None:
             impl_processor = self.impl_instance.literal_processor(dialect)
             if impl_processor:
-
                 fixed_impl_processor = impl_processor
                 fixed_process_literal_param = process_literal_param
 
@@ -2140,7 +2161,6 @@ class TypeDecorator(SchemaEventTarget, ExternalType, TypeEngine[_T]):
 
     @util.memoized_property
     def _has_bind_expression(self) -> bool:
-
         return (
             util.method_is_overridden(self, TypeDecorator.bind_expression)
             or self.impl_instance._has_bind_expression
@@ -2229,7 +2249,7 @@ class TypeDecorator(SchemaEventTarget, ExternalType, TypeEngine[_T]):
         """
         return self
 
-    def copy(self: SelfTypeDecorator, **kw: Any) -> SelfTypeDecorator:
+    def copy(self, **kw: Any) -> Self:
         """Produce a copy of this :class:`.TypeDecorator` instance.
 
         This is a shallow copy and is provided to fulfill part of
