@@ -13,20 +13,20 @@ unpickling.
 
 from __future__ import annotations
 
-from . import fixtures
+from .entities import ComparableEntity
 from ..schema import Column
 from ..types import String
 
 
-class User(fixtures.ComparableEntity):
+class User(ComparableEntity):
     pass
 
 
-class Order(fixtures.ComparableEntity):
+class Order(ComparableEntity):
     pass
 
 
-class Dingaling(fixtures.ComparableEntity):
+class Dingaling(ComparableEntity):
     pass
 
 
@@ -34,20 +34,20 @@ class EmailUser(User):
     pass
 
 
-class Address(fixtures.ComparableEntity):
+class Address(ComparableEntity):
     pass
 
 
 # TODO: these are kind of arbitrary....
-class Child1(fixtures.ComparableEntity):
+class Child1(ComparableEntity):
     pass
 
 
-class Child2(fixtures.ComparableEntity):
+class Child2(ComparableEntity):
     pass
 
 
-class Parent(fixtures.ComparableEntity):
+class Parent(ComparableEntity):
     pass
 
 
@@ -61,7 +61,7 @@ class Mixin:
     email_address = Column(String)
 
 
-class AddressWMixin(Mixin, fixtures.ComparableEntity):
+class AddressWMixin(Mixin, ComparableEntity):
     pass
 
 
